@@ -1,6 +1,6 @@
 # -*- coding: utf-8  -*-
 #
-# Copyright (C) 2012-2015 Ben Kurtovic <ben.kurtovic@gmail.com>
+# Copyright (C) 2012-2018 Ben Kurtovic <ben.kurtovic@gmail.com>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -65,7 +65,6 @@ class Tokenizer(object):
     MARKERS = ["{", "}", "[", "]", "<", ">", "|", "=", "&", "'", "#", "*", ";",
                ":", "/", "-", "!", "\n", START, END]
     MAX_DEPTH = 40
-    MAX_CYCLES = 100000
     regex = re.compile(r"([{}\[\]<>|=&'#*;:/\\\"\-!\n])", flags=re.IGNORECASE)
     tag_splitter = re.compile(r"([\s\"\'\\]+)")
 
@@ -75,7 +74,8 @@ class Tokenizer(object):
         self._stacks = []
         self._global = 0
         self._depth = 0
-        self._cycles = 0
+        self._bad_routes = set()
+        self._skip_style_tags = False
 
     @property
     def _stack(self):
@@ -100,11 +100,24 @@ class Tokenizer(object):
     def _textbuffer(self, value):
         self._stacks[-1][2] = value
 
+    @property
+    def _stack_ident(self):
+        """An identifier for the current stack.
+
+        This is based on the starting head position and context. Stacks with
+        the same identifier are always parsed in the same way. This can be used
+        to cache intermediate parsing info.
+        """
+        return self._stacks[-1][3]
+
     def _push(self, context=0):
         """Add a new token stack, context, and textbuffer to the list."""
-        self._stacks.append([[], context, []])
+        new_ident = (self._head, context)
+        if new_ident in self._bad_routes:
+            raise BadRoute(context)
+
+        self._stacks.append([[], context, [], new_ident])
         self._depth += 1
-        self._cycles += 1
 
     def _push_textbuffer(self):
         """Push the textbuffer onto the stack as a Text node and clear it."""
@@ -129,7 +142,15 @@ class Tokenizer(object):
 
     def _can_recurse(self):
         """Return whether or not our max recursion depth has been exceeded."""
-        return self._depth < self.MAX_DEPTH and self._cycles < self.MAX_CYCLES
+        return self._depth < self.MAX_DEPTH
+
+    def _memoize_bad_route(self):
+        """Remember that the current route (head + context at push) is invalid.
+
+        This will be noticed when calling _push with the same head and context,
+        and the route will be failed immediately.
+        """
+        self._bad_routes.add(self._stack_ident)
 
     def _fail_route(self):
         """Fail the current tokenization route.
@@ -138,6 +159,7 @@ class Tokenizer(object):
         :exc:`.BadRoute`.
         """
         context = self._context
+        self._memoize_bad_route()
         self._pop()
         raise BadRoute(context)
 
@@ -264,14 +286,14 @@ class Tokenizer(object):
         elif self._context & contexts.TEMPLATE_PARAM_VALUE:
             self._context ^= contexts.TEMPLATE_PARAM_VALUE
         else:
-            self._emit_all(self._pop(keep_context=True))
+            self._emit_all(self._pop())
         self._context |= contexts.TEMPLATE_PARAM_KEY
         self._emit(tokens.TemplateParamSeparator())
         self._push(self._context)
 
     def _handle_template_param_value(self):
         """Handle a template parameter's value at the head of the string."""
-        self._emit_all(self._pop(keep_context=True))
+        self._emit_all(self._pop())
         self._context ^= contexts.TEMPLATE_PARAM_KEY
         self._context |= contexts.TEMPLATE_PARAM_VALUE
         self._emit(tokens.TemplateParamEquals())
@@ -282,7 +304,7 @@ class Tokenizer(object):
             if not self._context & (contexts.HAS_TEXT | contexts.HAS_TEMPLATE):
                 self._fail_route()
         elif self._context & contexts.TEMPLATE_PARAM_KEY:
-            self._emit_all(self._pop(keep_context=True))
+            self._emit_all(self._pop())
         self._head += 1
         return self._pop()
 
@@ -299,17 +321,34 @@ class Tokenizer(object):
 
     def _parse_wikilink(self):
         """Parse an internal wikilink at the head of the wikicode string."""
+        reset = self._head + 1
         self._head += 2
-        reset = self._head - 1
         try:
-            wikilink = self._parse(contexts.WIKILINK_TITLE)
+            # If the wikilink looks like an external link, parse it as such:
+            link, extra, delta = self._really_parse_external_link(True)
         except BadRoute:
-            self._head = reset
-            self._emit_text("[[")
+            self._head = reset + 1
+            try:
+                # Otherwise, actually parse it as a wikilink:
+                wikilink = self._parse(contexts.WIKILINK_TITLE)
+            except BadRoute:
+                self._head = reset
+                self._emit_text("[[")
+            else:
+                self._emit(tokens.WikilinkOpen())
+                self._emit_all(wikilink)
+                self._emit(tokens.WikilinkClose())
         else:
-            self._emit(tokens.WikilinkOpen())
-            self._emit_all(wikilink)
-            self._emit(tokens.WikilinkClose())
+            if self._context & contexts.EXT_LINK_TITLE:
+                # In this exceptional case, an external link that looks like a
+                # wikilink inside of an external link is parsed as text:
+                self._head = reset
+                self._emit_text("[[")
+                return
+            self._emit_text("[")
+            self._emit(tokens.ExternalLinkOpen(brackets=True))
+            self._emit_all(link)
+            self._emit(tokens.ExternalLinkClose())
 
     def _handle_wikilink_separator(self):
         """Handle the separator between a wikilink's title and its text."""
@@ -475,12 +514,16 @@ class Tokenizer(object):
 
     def _parse_external_link(self, brackets):
         """Parse an external link at the head of the wikicode string."""
+        if self._context & contexts.NO_EXT_LINKS or not self._can_recurse():
+            if not brackets and self._context & contexts.DL_TERM:
+                self._handle_dl_term()
+            else:
+                self._emit_text(self._read())
+            return
+
         reset = self._head
         self._head += 1
         try:
-            bad_context = self._context & contexts.NO_EXT_LINKS
-            if bad_context or not self._can_recurse():
-                raise BadRoute()
             link, extra, delta = self._really_parse_external_link(brackets)
         except BadRoute:
             self._head = reset
@@ -592,8 +635,8 @@ class Tokenizer(object):
     def _parse_entity(self):
         """Parse an HTML entity at the head of the wikicode string."""
         reset = self._head
-        self._push()
         try:
+            self._push(contexts.HTML_ENTITY)
             self._really_parse_entity()
         except BadRoute:
             self._head = reset
@@ -633,8 +676,9 @@ class Tokenizer(object):
             self._emit_first(tokens.TagAttrQuote(char=data.quoter))
             self._emit_all(self._pop())
         buf = data.padding_buffer
-        self._emit_first(tokens.TagAttrStart(pad_first=buf["first"],
-            pad_before_eq=buf["before_eq"], pad_after_eq=buf["after_eq"]))
+        self._emit_first(tokens.TagAttrStart(
+            pad_first=buf["first"], pad_before_eq=buf["before_eq"],
+            pad_after_eq=buf["after_eq"]))
         self._emit_all(self._pop())
         for key in data.padding_buffer:
             data.padding_buffer[key] = ""
@@ -687,6 +731,7 @@ class Tokenizer(object):
             elif data.context & data.CX_NOTE_SPACE:
                 if data.context & data.CX_QUOTED:
                     data.context = data.CX_ATTR_VALUE
+                    self._memoize_bad_route()
                     self._pop()
                     self._head = data.reset - 1  # Will be auto-incremented
                     return  # Break early
@@ -711,7 +756,13 @@ class Tokenizer(object):
                         data.context |= data.CX_QUOTED
                         data.quoter = chunk
                         data.reset = self._head
-                        self._push(self._context)
+                        try:
+                            self._push(self._context)
+                        except BadRoute:
+                            # Already failed to parse this as a quoted string
+                            data.context = data.CX_ATTR_VALUE
+                            self._head -= 1
+                            return
                         continue
                 elif data.context & data.CX_QUOTED:
                     if chunk == data.quoter and not escaped:
@@ -787,6 +838,12 @@ class Tokenizer(object):
                 depth -= 1
                 if depth == 0:
                     break
+            elif isinstance(token, tokens.TagCloseSelfclose):
+                depth -= 1
+                if depth == 0:  # pragma: no cover (untestable/exceptional)
+                    raise ParserError(
+                        "_handle_single_tag_end() got an unexpected "
+                        "TagCloseSelfclose")
         else:  # pragma: no cover (untestable/exceptional case)
             raise ParserError("_handle_single_tag_end() missed a TagCloseOpen")
         padding = stack[index].padding
@@ -807,6 +864,7 @@ class Tokenizer(object):
                     if data.context & data.CX_QUOTED:
                         # Unclosed attribute quote: reset, don't die
                         data.context = data.CX_ATTR_VALUE
+                        self._memoize_bad_route()
                         self._pop()
                         self._head = data.reset
                         continue
@@ -1046,6 +1104,7 @@ class Tokenizer(object):
                     if data.context & data.CX_QUOTED:
                         # Unclosed attribute quote: reset, don't die
                         data.context = data.CX_ATTR_VALUE
+                        self._memoize_bad_route()
                         self._pop()
                         self._head = data.reset
                         continue
@@ -1057,23 +1116,26 @@ class Tokenizer(object):
 
     def _parse_table(self):
         """Parse a wikicode table by starting with the first line."""
-        reset = self._head + 1
+        reset = self._head
         self._head += 2
-        self._push(contexts.TABLE_OPEN)
         try:
+            self._push(contexts.TABLE_OPEN)
             padding = self._handle_table_style("\n")
         except BadRoute:
             self._head = reset
-            self._emit_text("{|")
+            self._emit_text("{")
             return
         style = self._pop()
 
         self._head += 1
+        restore_point = self._stack_ident
         try:
             table = self._parse(contexts.TABLE_OPEN)
         except BadRoute:
+            while self._stack_ident != restore_point:
+                self._pop()
             self._head = reset
-            self._emit_text("{|")
+            self._emit_text("{")
             return
 
         self._emit_table_tag("{|", "table", style, padding, None, table, "|}")
@@ -1089,11 +1151,7 @@ class Tokenizer(object):
             return
 
         self._push(contexts.TABLE_OPEN | contexts.TABLE_ROW_OPEN)
-        try:
-            padding = self._handle_table_style("\n")
-        except BadRoute:
-            self._pop()
-            raise
+        padding = self._handle_table_style("\n")
         style = self._pop()
 
         # Don't parse the style separator:
@@ -1321,21 +1379,23 @@ class Tokenizer(object):
                 if result is not None:
                     return result
             elif self._read(-1) in ("\n", self.START) and this in ("#", "*", ";", ":"):
-                    self._handle_list()
-            elif self._read(-1) in ("\n", self.START) and this == next == self._read(2) == self._read(3) == "-":
-                    self._handle_hr()
+                self._handle_list()
+            elif self._read(-1) in ("\n", self.START) and (
+                    this == next == self._read(2) == self._read(3) == "-"):
+                self._handle_hr()
             elif this in ("\n", ":") and self._context & contexts.DL_TERM:
                 self._handle_dl_term()
                 if this == "\n":
                     # Kill potential table contexts
                     self._context &= ~contexts.TABLE_CELL_LINE_CONTEXTS
             # Start of table parsing
-            elif this == "{" and next == "|" and (self._read(-1) in ("\n", self.START) or
+            elif this == "{" and next == "|" and (
+                    self._read(-1) in ("\n", self.START) or
                     (self._read(-2) in ("\n", self.START) and self._read(-1).isspace())):
                 if self._can_recurse():
                     self._parse_table()
                 else:
-                    self._emit_text("{|")
+                    self._emit_text("{")
             elif self._context & contexts.TABLE_OPEN:
                 if this == next == "|" and self._context & contexts.TABLE_TD_LINE:
                     if self._context & contexts.TABLE_CELL_OPEN:
@@ -1356,7 +1416,7 @@ class Tokenizer(object):
                     self._context &= ~contexts.TABLE_CELL_LINE_CONTEXTS
                     self._emit_text(this)
                 elif (self._read(-1) in ("\n", self.START) or
-                    (self._read(-2) in ("\n", self.START) and self._read(-1).isspace())):
+                      (self._read(-2) in ("\n", self.START) and self._read(-1).isspace())):
                     if this == "|" and next == "}":
                         if self._context & contexts.TABLE_CELL_OPEN:
                             return self._handle_table_cell_end()
@@ -1388,10 +1448,12 @@ class Tokenizer(object):
 
     def tokenize(self, text, context=0, skip_style_tags=False):
         """Build a list of tokens from a string of wikicode and return it."""
-        self._skip_style_tags = skip_style_tags
         split = self.regex.split(text)
         self._text = [segment for segment in split if segment]
-        self._head = self._global = self._depth = self._cycles = 0
+        self._head = self._global = self._depth = 0
+        self._bad_routes = set()
+        self._skip_style_tags = skip_style_tags
+
         try:
             tokens = self._parse(context)
         except BadRoute:  # pragma: no cover (untestable/exceptional case)
