@@ -23,11 +23,7 @@
 from __future__ import print_function, unicode_literals
 import json
 import os
-
-try:
-    import unittest2 as unittest
-except ImportError:
-    import unittest
+import unittest
 
 import mwparserfromhell
 from mwparserfromhell.compat import py3k, str
